@@ -959,6 +959,15 @@ bool
 cli_common_getenv_pgsetup(PostgresSetup *pgSetup,
 						  SSLCommandLineOptions *sslCommandLineOptions)
 {
+	if (env_exists("PGDATABASE"))
+	{
+		if (!get_env_copy("PGDATABASE", pgSetup->dbname, sizeof(pgSetup->dbname)))
+		{
+			/* errors have already been logged */
+			exit(EXIT_CODE_BAD_ARGS);
+		}
+	}
+
 	if (env_exists(PG_AUTOCTL_SSL_SELF_SIGNED))
 	{
 		char selfSigned[BUFSIZE] = { 0 };
@@ -1515,7 +1524,7 @@ keeper_cli_print_version(int argc, char **argv)
 				"pg_autoctl extension version %s\n",
 				PG_AUTOCTL_EXTENSION_VERSION);
 		fformat(stdout, "compiled with %s\n", PG_VERSION_STR);
-		fformat(stdout, "compatible with Postgres 10, 11, 12, 13, and 14\n");
+		fformat(stdout, "compatible with Postgres 13, 14, 15, 16 and 17\n");
 	}
 
 	exit(0);
