@@ -2249,7 +2249,7 @@ pgsql_create_user(PGSQL *pgsql, const char *userName, const char *password,
 	if (password)
 	{
 		/* show the statement before we append the password */
-		log_debug("Running command on Postgres: %s PASSWORD '*****';", query->data);
+		log_debug("%s PASSWORD '*****';", query->data);
 
 		escapedIdentifier = PQescapeLiteral(connection, password, strlen(password));
 		if (escapedIdentifier == NULL)
@@ -2267,7 +2267,7 @@ pgsql_create_user(PGSQL *pgsql, const char *userName, const char *password,
 	}
 	else
 	{
-		log_debug("Running command on Postgres: %s;", query->data);
+		log_debug("%s;", query->data);
 	}
 
 	/* memory allocation could have failed while building string */
@@ -3094,18 +3094,18 @@ bool
 parseTimeLineHistory(const char *filename, const char *content,
 					 IdentifySystem *system)
 {
-	char *historyLines[BUFSIZE] = { 0 };
-	int lineCount = splitLines((char *) content, historyLines, BUFSIZE);
+	int lineCount = countLines((char *) content);
+	char **historyLines = (char **) calloc(lineCount, sizeof(char *));
 	int lineNumber = 0;
 
-	if (lineCount >= PG_AUTOCTL_MAX_TIMELINES)
+	if (historyLines == NULL)
 	{
-		log_error("history file \"%s\" contains %d lines, "
-				  "pg_autoctl only supports up to %d lines",
-				  filename, lineCount, PG_AUTOCTL_MAX_TIMELINES - 1);
+		log_error(ALLOCATION_FAILED_ERROR);
 		return false;
 	}
 
+	splitLines((char *) content, historyLines, lineCount);
+
 	uint64_t prevend = InvalidXLogRecPtr;
 
 	system->timelines.count = 0;
@@ -3141,6 +3141,7 @@ parseTimeLineHistory(const char *filename, const char *content,
 		{
 			log_error("Failed to parse history file line %d: \"%s\"",
 					  lineNumber, ptr);
+			free(historyLines);
 			return false;
 		}
 
@@ -3149,6 +3150,7 @@ parseTimeLineHistory(const char *filename, const char *content,
 		if (!stringToUInt(historyLines[lineNumber], &(entry->tli)))
 		{
 			log_error("Failed to parse history timeline \"%s\"", tabptr);
+			free(historyLines);
 			return false;
 		}
 
@@ -3167,6 +3169,7 @@ parseTimeLineHistory(const char *filename, const char *content,
 		{
 			log_error("Failed to parse history timeline %d LSN \"%s\"",
 					  entry->tli, lsn);
+			free(historyLines);
 			return false;
 		}
 
@@ -3184,6 +3187,8 @@ parseTimeLineHistory(const char *filename, const char *content,
 		entry = &(system->timelines.history[++system->timelines.count]);
 	}
 
+	free(historyLines);
+
 	/*
 	 * Create one more entry for the "tip" of the timeline, which has no entry
 	 * in the history file.
