@@ -124,12 +124,14 @@ StartMonitorNode(void)
 	DefineCustomBoolVariable("pgautofailover.enable_version_checks",
 							 "Enable extension version compatibility checks",
 							 NULL, &EnableVersionChecks, true, PGC_SIGHUP,
-							 GUC_NO_SHOW_ALL, NULL, NULL, NULL);
+							 GUC_NO_SHOW_ALL | GUC_NOT_IN_SAMPLE,
+							 NULL, NULL, NULL);
 
 	DefineCustomBoolVariable("pgautofailover.enable_health_checks",
 							 "Enable background health checks",
 							 NULL, &HealthChecksEnabled, true, PGC_SIGHUP,
-							 GUC_NO_SHOW_ALL, NULL, NULL, NULL);
+							 GUC_NO_SHOW_ALL | GUC_NOT_IN_SAMPLE,
+							 NULL, NULL, NULL);
 
 	DefineCustomIntVariable("pgautofailover.health_check_period",
 							"Duration between each check (in milliseconds).",
@@ -193,6 +195,7 @@ StartMonitorNode(void)
 	strlcpy(worker.bgw_name, "pg_auto_failover monitor", sizeof(worker.bgw_name));
 	strlcpy(worker.bgw_function_name, "HealthCheckWorkerLauncherMain",
 			sizeof(worker.bgw_function_name));
+	strlcpy(worker.bgw_type, "pgautofailover", sizeof(worker.bgw_type));
 
 	RegisterBackgroundWorker(&worker);
 }
