@@ -92,11 +92,34 @@ def test_000_create_monitor():
     )
     serverCert.create_signed_certificate(cluster.cert)
 
+    # the root user also needs the certificates, tests are connecting with it
+    root_top_directory = "/root/.postgresql"
+    p = subprocess.run(
+        ["sudo", "install", "-d", "-m", "740", root_top_directory]
+    )
+    assert p.returncode == 0
+
+    p = subprocess.run(
+        [
+            "sudo",
+            "cp",
+            clientCert.crt,
+            clientCert.csr,
+            clientCert.key,
+            root_top_directory,
+        ]
+    )
+    assert p.returncode == 0
+
     p = subprocess.run(
         [
             "ls",
             "-ld",
             client_top_directory,
+            root_top_directory,
+            os.path.join(root_top_directory, "postgresql.crt"),
+            os.path.join(root_top_directory, "postgresql.csr"),
+            os.path.join(root_top_directory, "postgresql.key"),
             cluster.cert.crt,
             cluster.cert.csr,
             cluster.cert.key,
@@ -112,10 +135,6 @@ def test_000_create_monitor():
     )
     print("%s" % p.stdout)
 
-    # the root user also needs the certificates, tests are connecting with it
-    subprocess.run(["ln", "-s", client_top_directory, "/root/.postgresql"])
-    assert p.returncode == 0
-
     #
     # Now create the monitor Postgres instance with the certificates
     #
